import logging
from typing import Dict

import requests
import requests.packages

from basecamp_chatbot.exceptions import BasecampApiException
from basecamp_chatbot.models import Result


class RestAdapter:
    def __init__(
        self,
        bot_url: str,
        ssl_verify: bool = True,
        logger: logging.Logger = None,
    ):
        """
        :param bot_url: URL to POST updates to. See README.md for more details
        :param ssl_verify: Normally set to True, but if having SSL/TLS cert validation issues, can turn off with False
        :param logger: (optional) If your app has a logger, pass it in here
        """
        self.url = bot_url
        self._ssl_verify = ssl_verify
        self._logger = logger or logging.getLogger(__name__)
        if not ssl_verify:
            requests.packages.urllib3.disable_warnings()

    def post(self, data: Dict) -> Result:
        log_line_pre = f"url={self.url}"
        # Log HTTP params and perform an HTTP request, catching and re-raising any exceptions
        try:
            self._logger.debug(msg=log_line_pre)
            response = requests.request(
                method="POST",
                url=self.url,
                verify=self._ssl_verify,
                json=data,
            )
        except requests.exceptions.RequestException as e:
            self._logger.error(msg=(str(e)))
            raise BasecampApiException("Request failed") from e
        # Deserialize JSON output to Python object, or return failed Result on exception

        is_success = response.status_code == 201
        log_line = f"{log_line_pre}, success={is_success}, status_code={response.status_code}, message={response.reason}"

        if is_success:
            self._logger.debug(msg=log_line)
            return Result(response.status_code, message=response.reason)
        self._logger.error(msg=log_line)
        raise BasecampApiException(f"{response.status_code}: {response.reason}")



#!/usr/bin/env python

"""
Post a string to the specified Basecamp chatbot

Usage:
  bc_post_from_str <bot_url> <message>
"""

from docopt import docopt
from basecamp_chatbot.chatbot import Chatbot


def _post_string(bot_url, message):
    bot = Chatbot(bot_url)
    bot.post_string(message)


def _main():
    args = docopt(__doc__)
    _post_string(args["<bot_url>"], args["<message>"])


if __name__ == "__main__":
    _main()



#!/usr/bin/env python

"""
Post the contents of a file to the specified Basecamp chatbot

Usage:
  bc_post_from_file <bot_url> <content_file>
"""

from pathlib import Path

from docopt import docopt
from basecamp_chatbot.chatbot import Chatbot


def _post_file(bot_url, content_file):
    bot = Chatbot(bot_url)
    bot.post_file(Path(content_file))


def _main():
    args = docopt(__doc__)
    _post_file(args["<bot_url>"], args["<content_file>"])


if __name__ == "__main__":
    _main()



from typing import List, Dict


class Result:
    def __init__(self, status_code: int, message: str = "", data: List[Dict] = None):
        """
        Result returned from low-level RestAdapter
        :param status_code: Standard HTTP Status code
        :param message: Human readable result
        :param data: Python List of Dicts
        """
        self.status_code = int(status_code)
        self.message = str(message)
        self.data = data if data else []



import logging

from basecamp_chatbot.rest_adapter import RestAdapter
from pathlib import Path


class Chatbot:
    def __init__(
        self,
        bot_url: str,
        ssl_verify: bool = True,
        logger: logging.Logger = None,
    ):
        self._rest_adapter = RestAdapter(bot_url, ssl_verify, logger)

    def post_string(self, message: str):
        self._rest_adapter.post({"content": message})

    def post_file(self, filepath: Path):
        with open(filepath, "r") as file:
            message = file.read()
            self.post_string(message)
